package models

import (
	"encoding/json"
	"net/http"
	"strconv"
	"strings"

	"forge.grandlyon.com/apoyen/elections/internal/auth"
)

func (d *DataHandler) handleCandidateList(w http.ResponseWriter, r *http.Request) {
	id, _ := strconv.Atoi(strings.TrimPrefix(r.URL.Path, "/api/CandidateList/"))
	switch method := r.Method; method {
	case "GET":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN", "CAPTURER", "VISUALIZER":
			d.getCandidateList(w, r, id)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	case "POST":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN":
			d.postCandidateList(w, r)
		case "CAPTURER", "VISUALIZER":
			http.Error(w, ErrorNotAuthorizeMethodOnRessource, http.StatusMethodNotAllowed)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}

	case "PUT":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN":
			d.putCandidateList(w, r, id)
		case "CAPTURER", "VISUALIZER":
			http.Error(w, ErrorNotAuthorizeMethodOnRessource, http.StatusMethodNotAllowed)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	case "DELETE":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN":
			d.deleteCandidateList(w, r, id)
		case "CAPTURER", "VISUALIZER":
			http.Error(w, ErrorNotAuthorizeMethodOnRessource, http.StatusMethodNotAllowed)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	default:
		http.Error(w, "method not allowed", 400)
	}
}

func (d *DataHandler) getCandidateList(w http.ResponseWriter, r *http.Request, id int) {
	if id != 0 {
		var o CandidateList
		if err := d.db.Preload("Candidates").Preload("Votes").First(&o, id).Error; err != nil {
			http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
			return
		}
		json.NewEncoder(w).Encode(o)
	} else {
		var o []CandidateList
		d.db.Preload("Candidates").Preload("Votes").Find(&o)
		json.NewEncoder(w).Encode(o)
	}
}

func (d *DataHandler) postCandidateList(w http.ResponseWriter, r *http.Request) {
	var o CandidateList
	err := json.NewDecoder(r.Body).Decode(&o)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	// Check that AreaID exist
	var area Area
	if err := d.db.First(&area, o.AreaID).Error; err != nil {
		http.Error(w, ErrorParentNotFound, http.StatusInternalServerError)
		return
	}

	d.db.Create(&o)
	d.db.Last(&o)
	json.NewEncoder(w).Encode(o)

}

func (d *DataHandler) putCandidateList(w http.ResponseWriter, r *http.Request, id int) {
	var o CandidateList
	if err := d.db.Preload("Candidates").Preload("Votes").First(&o, id).Error; err != nil {
		http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
		return
	}
	var candidateList CandidateList
	err := json.NewDecoder(r.Body).Decode(&candidateList)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	o.Name = candidateList.Name
	o.PartyID = candidateList.PartyID
	o.RoundID = candidateList.RoundID
	o.AreaID = candidateList.AreaID
	d.db.Save(&o)
	json.NewEncoder(w).Encode(o)

}

func (d *DataHandler) deleteCandidateList(w http.ResponseWriter, r *http.Request, id int) {
	if id != 0 {
		var o CandidateList
		if err := d.db.Preload("Candidates").Preload("Votes").First(&o, id).Error; err != nil {
			http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
			return
		}

		// TODO remove candidate in cascade

		d.db.Delete(&o)
	} else {
		http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
	}
}
