import * as Messages from "/services/messages/messages.js";

let deskRoundModel;

export function getDeskRoundModel() {
  if (deskRoundModel == null) {
    deskRoundModel = new DeskRoundModel();
  }
  return deskRoundModel;
}

class DeskRoundModel {
  constructor() {}

  async getDeskRound(id) {
    if (this.deskRounds == null) await this.refreshDeskRounds();
    let deskRoundToGet;
    this.deskRounds.forEach((deskRound) => {
      if (deskRound.ID == id) deskRoundToGet = deskRound;
    });
    return deskRoundToGet;
  }

  async getDeskRounds() {
    if (this.deskRounds == null) {
      try {
        const response = await fetch("/api/DeskRound/", {
          method: "GET",
          headers: new Headers({
            "XSRF-Token": this.current_user.xsrftoken,
          }),
        });
        if (response.status !== 200) {
          throw new Error(
            `DeskRounds could not be fetched (status ${response.status})`
          );
        }
        this.deskRounds = await response.json();
      } catch (e) {
        Messages.Show("is-warning", e.message);
        console.error(e);
      }
    }
    return this.deskRounds;
  }

  async saveDeskRound(ID, Validated) {
    try {
      const response = await fetch("/api/DeskRound/" + ID, {
        method: "PUT",
        headers: new Headers({
          "XSRF-Token": this.current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: ID,
          Validated: Validated,
        }),
      });
      if (response.status !== 200) {
        throw new Error(
          `DeskRound could not be updated or created (status ${response.status})`
        );
      }
      this.refreshDeskRounds();
      Messages.Show("is-success", "Bureau de vote mis à jour");
      return await response.json();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
      return;
    }
  }

  async refreshDeskRounds() {
    this.deskRounds = null;
    await this.getDeskRounds();
  }
}
